/**
 * View that shows the entire post data 
 * (e.g.: in the details page)
 * 
 * @author Matanya
 */
var FullPostView = Backbone.View.extend(
{	
	tagName: 'div',

	id: 'details_' + Utils.Helpers.generateGUID(),

	events: 
	{
		'tap .read_more_button': '_getMoreClick',
		'tap .video_img_container': 'videoClicked'
	},
	
	initialize: function () 
	{
		$(this.el).attr('data-role', 'page');		

		//this.model.bind('change:message', this.render, this);
		this.model.bind('change:commentsCount', this.renderCommentsCount, this);
		
		var commentsCollection = this.model.get('comments');
 		commentsCollection.bind('add', this._renderOneComment, this);
 		commentsCollection.bind('getCommentsSuccess', this.renderShowMore, this);
 		commentsCollection.bind('getCommentsFail', this.renderShowMore, this);
 		commentsCollection.bind('getCommentsFail', this.onGetDataFail, this);
 		commentsCollection.bind('getNextComments', this.renderShowMore, this);
		this.model.bind('change:eof', this.renderShowMore, this);
		
		// Go to FB and get the comments:
		this.model.getComments();
	},
	
	render: function() 
	{	
		var $thisEl = $(this.el);
		$thisEl.addClass('facebook_page');

		var timeAgo = Utils.TimeDate.timeAsText(this.model.get('createdTime') / 1000);

		var type = this.model.get('type');
		var picture = this.model.get('picture');
		var pictureHtml = "";
		if (picture)
		{
			if (type === "video") 
			{
				pictureHtml ='<div class="video_img_container">' + 
								'<span class="play_icon"></span>' +
								'<img class="post_list_img video_img" src="' + picture + '"></img>' +
							 '</div>';
			}
			else	
			{
				pictureHtml = '<img class="post_list_img photo_img" src="' + picture + '"></img>';
			}
		}

		// TODO: change to templates:
		var html =  
				'<div class="post_full_item item_padding">' +
					'<img class="post_from_img" src="' + this.model.get('fromPicture') +'"></img>' + 
					'<div class="post_data_container">' +
					 	'<div class="post_from_name title_color">' + this.model.get('fromName') + '</div>' + 
						'<p class="post_list_message text_color">' +
							this.model.get('message') +
						'</p>' +
						pictureHtml + 
						'<div class="post_list_extra_info text2_color ">' +
						// TODO:Do we want to use this icon? 
							'<span class="comments_count"></span>' +
							'<span class="facebook_icon"></span>' +												
							'<span class="time_ago">' + timeAgo + '</span>' +			
						'</div>' +	
					'</div>' +
				'</div>' +
				'<div class="comments_section">' + 
					'<div class="comments_separator">' + 
						'<div class="comments_separator_icon"></div>' +
					'</div>' +
				'</div>';

		$thisEl.html(html);

		// Update the scroller after loading image:
		var $picture = $thisEl.find('.post_list_img');
		$picture.load(function()
		{
			Scrolling.onContentChanged();
		});

		this.$commentsSection = $thisEl.find('.comments_section');

		// Render the "comments count" element:
		this.$commentsCount = $thisEl.find('.comments_count');
		this.renderCommentsCount();
		
		//Add the "comments list" element + render it
		this.$commentsList = $('<ul class="post_comments"></ul>');
		this.$commentsSection.append(this.$commentsList);
		this.renderComments();

		//Add the get more comments "button" / "loading-indicator" + render it
		this.$getMoreContainer = $('<a class="more_container button">' +
									'<span class="button_text">' +
										'Show more comments' +
									'</span>' +
									'<span class="loading_text">' +
										'<span class="loading_icon spin"></span>' + 
										'  Loading comments...' +
									'</span>' +
								'</div>');
								
		this.$commentsSection.append(this.$getMoreContainer);
		this.$getMoreButton = this.$getMoreContainer.find('.button_text');
		this.$getMoreLoading = this.$getMoreContainer.find('.loading_text');
		this.renderShowMore();

		return this;
		
	},

	renderCommentsCount: function()
	{
		if (!this.$commentsCount)
		{
			return;
		}
		var commentsCount = this.model.get('commentsCount');
		if (commentsCount > 0)
		{
			this.$commentsSection.show();
		}
		else
		{
			this.$commentsSection.hide();
		}
		
		this.$commentsCount.html(localization.formatNumber(commentsCount) +
								' comments');
	},
	
	renderComments: function() 
	{					
		var commentsCollection = this.model.get('comments');
		_.each(commentsCollection, this._renderOneComment.hitch(this)); 
		return this;
	},
	
	_renderOneComment: function(comment) 
	{					
		if (!this.$commentsList)
		{
			return;
		}

		var commentView = new CommentView({ model: comment });

		this.$commentsList.append(commentView.render().el);

		// TODO: Send only 1 trigger (e.g.: if we adds many items, hide "show more" etc.) 
		Scrolling.onContentChanged();
		
		return this;
	},

	/**
	 * NOTE: this is "show-more button" or the "show-more loading indicator"! 
	 */
	renderShowMore: function() 
	{
		if (!this.$getMoreContainer)
		{
			return;
		}

		if (this.model.get('eof')) 
		{
			this.$getMoreContainer.hide();
		}
		else
		{
			var commentsCollection = this.model.get('comments');
			if (commentsCollection.isLoading) 
			{
				this.$getMoreButton.hide();
				this.$getMoreLoading.show();
				this.$getMoreContainer.removeClass('read_more_button');
			}
			else
			{
				this.$getMoreLoading.hide();
				this.$getMoreButton.show();
				this.$getMoreContainer.addClass('read_more_button');
			}
			this.$getMoreContainer.show();
		}
		
		// TODO: Send only 1 trigger (e.g.: if we adds many items, hide "show more" etc.) 
		Scrolling.onContentChanged();		
	},

	onGetDataFail: function() 
	{
		cqm.showToastMessage('failed to get data');		
	},
	
	_getMoreClick: function() 
	{
		this.model.getNextComments();
	},
	
	videoClicked: function () 
	{
		this.model.setSelectedVideo(true);
	}	
	
	
});